#ifndef PODOFO_BASE_INCLUDES_H
#define PODOFO_BASE_INCLUDES_H

// Include common C headers
#include <cstdint>
#include <cstddef>
#include <cstring>

// Include common STL headers
#include <memory>
#include <string>
#include <string_view>
#include <vector>
#include <map>
#include <set>
#include <unordered_set>
#include <unordered_map>
#include <typeinfo>
#include <array>
#include <queue>
#include <functional>
#include <ios>
#include <exception>

// Include some common headers
#include "basetypes.h"
#include "EnumFlags.h"
#include "span.h"
#include "nullable.h"

#endif // PODOFO_BASE_INCLUDES
